'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createViewModelsReducer;

var _actions = require('./actions');

var _util = require('./util');

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function createViewModelsReducer() {
    var handlers = {};
    var initialState = {};
    var subscribers = {};

    handlers[_actions.PROVIDE_VIEW_MODELS] = function (state, _ref) {
        var viewModels = _ref.viewModels;

        delete handlers[_actions.PROVIDE_VIEW_MODELS];

        handlers[_actions.SUBSCRIBE] = function (state, _ref2) {
            var viewModel = _ref2.viewModel,
                aggregateId = _ref2.aggregateId;

            var key = (0, _util.getKey)(viewModel, aggregateId);

            if (subscribers[key]) {
                subscribers[key]++;
                return state;
            }

            subscribers[key] = 1;

            return _extends({}, state, _defineProperty({}, viewModel, _extends({}, state[viewModel], _defineProperty({}, aggregateId, viewModels.find(function (_ref3) {
                var name = _ref3.name;
                return viewModel === name;
            }).projection.Init()))));
        };

        handlers[_actions.UNSUBSCRIBE] = function (state, _ref4) {
            var viewModel = _ref4.viewModel,
                aggregateId = _ref4.aggregateId;

            var key = (0, _util.getKey)(viewModel, aggregateId);

            if (subscribers[key] > 1) {
                subscribers[key]--;
                return state;
            }

            subscribers[key] = 0;

            var _state$viewModel = state[viewModel],
                _ = _state$viewModel[aggregateId],
                nextViewModel = _objectWithoutProperties(_state$viewModel, [aggregateId]);

            return _extends({}, state, _defineProperty({}, viewModel, nextViewModel));
        };

        handlers[_actions.MERGE_STATE] = function (state, _ref5) {
            var viewModel = _ref5.viewModel,
                aggregateId = _ref5.aggregateId,
                actionState = _ref5.state;

            return _extends({}, state, _defineProperty({}, viewModel, _extends({}, state[viewModel], _defineProperty({}, aggregateId, actionState))));
        };

        viewModels.forEach(function (_ref6) {
            var viewModel = _ref6.name;

            initialState[viewModel] = {};
        });

        var map = viewModels.reduce(function (acc, _ref7) {
            var viewModel = _ref7.name,
                _ref7$projection = _ref7.projection,
                Init = _ref7$projection.Init,
                projection = _objectWithoutProperties(_ref7$projection, ['Init']);

            Object.keys(projection).forEach(function (eventType) {
                if (!acc[eventType]) {
                    acc[eventType] = {};
                }

                acc[eventType][viewModel] = projection[eventType];
            });
            return acc;
        }, {});

        Object.keys(map).forEach(function (eventType) {
            handlers[eventType] = function (state, action) {
                var nextState = _extends({}, state);

                Object.keys(map[eventType]).forEach(function (viewModel) {
                    var viewModelState = state[viewModel][action.aggregateId];

                    var result = map[eventType][viewModel](viewModelState, action);

                    nextState[viewModel] = _extends({}, nextState[viewModel], _defineProperty({}, action.aggregateId, result));
                });

                return nextState;
            };
        });

        return state;
    };

    return function () {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
        var action = arguments[1];

        var eventHandler = handlers[action.type];

        if (eventHandler) {
            return eventHandler(state, action);
        }

        return state;
    };
}