'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _actions = require('./actions');

var _actions2 = _interopRequireDefault(_actions);

var _socket = require('socket.io-client');

var _socket2 = _interopRequireDefault(_socket);

var _util = require('./util');

var _isomorphicFetch = require('isomorphic-fetch');

var _isomorphicFetch2 = _interopRequireDefault(_isomorphicFetch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var CRITICAL_LEVEL = 100;

var sendCommand = function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(command) {
        var response, text;
        return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        _context.next = 2;
                        return (0, _isomorphicFetch2.default)((0, _util.getRootableUrl)('/api/commands'), {
                            method: 'POST',
                            headers: { 'Content-Type': 'application/json' },
                            credentials: 'same-origin',
                            body: JSON.stringify(command)
                        });

                    case 2:
                        response = _context.sent;

                        if (!response.ok) {
                            _context.next = 5;
                            break;
                        }

                        return _context.abrupt('return', response.blob());

                    case 5:
                        _context.next = 7;
                        return response.text();

                    case 7:
                        text = _context.sent;

                        // eslint-disable-next-line no-console
                        console.error('Send command error:', text);
                        return _context.abrupt('return', Promise.reject(text));

                    case 10:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, undefined);
    }));

    return function sendCommand(_x) {
        return _ref.apply(this, arguments);
    };
}();

function createMiddleware(viewModels) {
    var subscribers = {
        viewModels: {},
        aggregateIds: {}
    };

    var requests = {};

    function getEventTypes() {
        var eventTypes = {};

        Object.keys(subscribers.viewModels).forEach(function (viewModel) {
            if (!subscribers.viewModels[viewModel]) {
                return;
            }

            var _viewModels$find$proj = viewModels.find(function (_ref2) {
                var name = _ref2.name;
                return name === viewModel;
            }).projection,
                Init = _viewModels$find$proj.Init,
                projection = _objectWithoutProperties(_viewModels$find$proj, ['Init']);

            Object.keys(projection).forEach(function (eventType) {
                eventTypes[eventType] = true;
            });
        });

        return Object.keys(eventTypes);
    }

    function getAggregateIds() {
        var aggregateIds = [];

        Object.keys(subscribers.aggregateIds).forEach(function (aggregateId) {
            if (!subscribers.aggregateIds[aggregateId]) {
                return;
            }
            aggregateIds.push(aggregateId);
        });

        return aggregateIds;
    }

    return function (store) {
        store.dispatch(_actions2.default.provideViewModels(viewModels));

        var socketIOFailCount = 0;
        var socketIO = null;

        var initSocketIO = function initSocketIO() {
            socketIO = (0, _socket2.default)(window.location.origin, {
                path: (0, _util.getRootableUrl)('/socket/')
            });

            socketIO.on('event', function (event) {
                return store.dispatch(JSON.parse(event));
            });

            socketIO.on('disconnect', function () {
                socketIOFailCount++;
                if (socketIOFailCount > CRITICAL_LEVEL) {
                    window.location.reload();
                }
                initSocketIO();
            });
        };

        initSocketIO();

        return function (next) {
            return function (action) {
                switch (action.type) {
                    case _actions.SUBSCRIBE:
                        {
                            var viewModel = action.viewModel,
                                aggregateId = action.aggregateId;


                            var needChange = !subscribers.viewModels[viewModel] || !subscribers.aggregateIds[aggregateId];

                            subscribers.viewModels[viewModel] = (subscribers.viewModels[viewModel] || 0) + 1;
                            subscribers.aggregateIds[aggregateId] = (subscribers.aggregateIds[aggregateId] || 0) + 1;

                            if (needChange) {
                                var key = (0, _util.getKey)(viewModel, aggregateId);
                                requests[key] = true;

                                (0, _isomorphicFetch2.default)((0, _util.getRootableUrl)('/api/query/' + viewModel + '?aggregateIds[]=' + viewModel), {
                                    method: 'GET',
                                    credentials: 'same-origin'
                                }).then(function (response) {
                                    if (response.ok) {
                                        return response.text();
                                    }
                                    throw new Error(response.text());
                                }).then(function (rawState) {
                                    var state = viewModels.find(function (_ref3) {
                                        var name = _ref3.name;
                                        return name === viewModel;
                                    }).deserializeState(rawState);

                                    if (subscribers.requests[key]) {
                                        delete requests[key];

                                        store.dispatch({
                                            type: _actions.MERGE_STATE,
                                            aggregateId: aggregateId,
                                            viewModel: viewModel,
                                            state: state
                                        });

                                        socketIO.emit('setSubscription', {
                                            types: getEventTypes(),
                                            ids: getAggregateIds()
                                        });
                                    }
                                });
                            }

                            break;
                        }
                    case _actions.UNSUBSCRIBE:
                        {
                            var _viewModel = action.viewModel,
                                _aggregateId = action.aggregateId;


                            subscribers.viewModels[_viewModel] = Math.max((subscribers.viewModels[_viewModel] || 0) - 1, 0);
                            subscribers.aggregateIds[_aggregateId] = Math.max((subscribers.aggregateIds[_aggregateId] || 0) - 1, 0);

                            var _needChange = !subscribers.viewModels[_viewModel] || !subscribers.aggregateIds[_aggregateId];

                            var _key = (0, _util.getKey)(_viewModel, _aggregateId);
                            delete requests[_key];

                            if (_needChange) {
                                socketIO.emit('setSubscription', {
                                    types: getEventTypes(),
                                    ids: getAggregateIds()
                                });
                            }

                            break;
                        }
                    case _actions.SEND_COMMAND:
                        {
                            var command = action.command,
                                _aggregateId2 = action.aggregateId,
                                aggregateName = action.aggregateName,
                                payload = action.payload;


                            if (command && (0, _util.checkRequiredFields)({ aggregateId: _aggregateId2, aggregateName: aggregateName }, 'Send command error:', JSON.stringify(action)) && !command.error) {
                                var normalizedCommand = {
                                    type: command.type,
                                    aggregateId: _aggregateId2,
                                    aggregateName: aggregateName,
                                    payload: payload
                                };

                                sendCommand(normalizedCommand, store.dispatch).catch(function (error) {
                                    store.dispatch(_extends({}, action, {
                                        command: _extends({}, action.command, {
                                            error: error
                                        })
                                    }));
                                });
                            }

                            break;
                        }
                    default:
                }

                return next(action);
            };
        };
    };
}

var middleware = typeof window === 'undefined' ? function () {
    return function () {
        return function (next) {
            return function (action) {
                return next(action);
            };
        };
    };
} : createMiddleware;

exports.default = middleware;